unit utlZoomStat;
//******************************************************************************
//         
//******************************************************************************
//        Author : Roman Romanov boss@small-pro.com
//       Project : IP76.RU' 2017
//      Web-site : small-pro.com
//   Description : TxZoomStatList -        
//                 DrawZoomStatList -  ,    
//                                    utlZoomText.GDrawZoomFuncList
//******************************************************************************

interface
uses Windows, Classes, SysUtils, Graphics
//-- GDI+ --------------------
   , GDIPAPI
   , GDIPOBJ
//-- this project -----------------------
   , utlZoomText 
   ;

type
//******************************************************************************
//       
//******************************************************************************
  PxZoomStat = ^TxZoomStat;
  TxZoomStat = packed record
    FIndex : Integer;
    FColor : TColor;
    FName  : string;
    FCount : Integer;
    FTime  : extended;
    FDiff  : extended;
    FTimeC : extended;
    FDiffC : extended;
    FTimeR : TRect;
    FDiffR : TRect;    
  end;

  TxZoomStatList = class
  private
    FOffX : Integer;
    FOffY : Integer;    
    FList : TList;
    FGDIPlus : boolean;
    function GetCount : Integer;
    function GetItem (Index : Integer) : PxZoomStat;
  public
    Constructor Create; virtual;
    Destructor Destroy; override;
    function Add (AIndex : Integer; AName : string; ATime, ADiff : Extended) : Integer; overload;
    function Add (AIndex : Integer; ATime, ADiff : Extended) : PxZoomStat; overload;    
    procedure Delete (Index : Integer);
    procedure Clear;
    property Count : Integer read GetCount;
    property Items[Index : Integer] : PxZoomStat read GetItem; default;
    //--------------------------------------------------------------------------
    property GDIPlus : boolean read FGDIPlus write FGDIPlus;
    property OffX : Integer read FOffX write FOffX;
    property OffY : Integer read FOffY write FOffY;    
  end;
  
//******************************************************************************
//     TxZoomStatList
//******************************************************************************
function xGZoomList (AWithCreate : boolean = true) : TxZoomStatList;  
//******************************************************************************
//   
//******************************************************************************
function DrawZoomStatList(ACanvas : TCanvas; ARect : TRect; 
                          AZoom, ASize : double; AText : string) : boolean;
  
implementation

var 
  GZoomList : TxZoomStatList = nil;
  
function xGZoomList (AWithCreate : boolean = true) : TxZoomStatList;
begin
  result := GZoomList;
  if (result <> nil) or (not AWithCreate) then exit;
  GZoomList := TxZoomStatList.Create;
  result := GZoomList;
end;

//******************************************************************************
// TxZoomStatList = class 
//******************************************************************************
Constructor TxZoomStatList.Create; 
begin
  FList := TList.Create;
  FGDIPlus := true;
end;

Destructor TxZoomStatList.Destroy;
begin
  Clear;
  Flist.Free;
  if GZoomList = self then 
    GZoomList := nil;
  inherited Destroy;
end;

function TxZoomStatList.GetCount : Integer;
begin
  result := FList.Count;
end;

function TxZoomStatList.GetItem (Index : Integer) : PxZoomStat;
begin
  if (Index < 0) or (Index>=FList.Count) then 
    result := nil
  else 
    result := FList[Index];
end;
  
function TxZoomStatList.Add (AIndex : Integer; AName : string; ATime, ADiff : Extended) : Integer;
var p : PxZoomStat;
begin
  result := -1;
  if (AIndex < 0 ) then exit;
  while AIndex >= Count do FList.Add(nil) ;
  if ADiff > 1 then ADiff := 2-ADiff;
  
  p := GetItem(AIndex);
  if p = nil then begin
    new (p);
    p^.FIndex := AIndex;
    p^.FColor := GColors[AIndex];
    if Aname <> '' then
      p^.FName := AName;
    p^.FCount := 0;
    p^.FTime := 0;    
    p^.FDiff := 0;
    p^.FTimeC := 0;    
    p^.FDiffC := 0;
    FList[AIndex] := p;
  end;
  if (ATime <> 0) or (ADiff <> 0) then begin
    p^.FCount:= p^.FCount+1;
    p^.FTime := p^.FTime + ATime;
    p^.FDiff := p^.FDiff + ADiff;  
  end;  
  result := p^.FIndex;
end;

function TxZoomStatList.Add (AIndex : Integer; ATime, ADiff : Extended) : PxZoomStat; 
begin
  Add (AIndex,'',ATime,ADiff);
  result := GetItem(AIndex);
end;
    
procedure TxZoomStatList.Delete (Index : Integer);
var p : PxZoomStat;
begin
  p := GetItem(Index);
  if p <> nil then begin
    Dispose(p);
  end;
  FList.Delete(Index);
end;

procedure TxZoomStatList.Clear;
begin
  while Count > 0 do Delete(Count-1);
  FList.Clear;
end;

function Darker(Color:TColor; Percent:Byte):TColor;
var   r,g,b:Byte;
begin
 Color:=ColorToRGB(Color);
 r:=GetRValue(Color);
 g:=GetGValue(Color);
 b:=GetBValue(Color);
 r:=r-muldiv(r,Percent,100);
 g:=g-muldiv(g,Percent,100);
 b:=b-muldiv(b,Percent,100);
 result:=RGB(r,g,b);
end;

function Lighter(Color:TColor; Percent:Byte):TColor;
var   r,g,b:Byte;
begin
 Color:=ColorToRGB(Color);
 r:=GetRValue(Color);
 g:=GetGValue(Color);
 b:=GetBValue(Color);
 r:=r+muldiv(255-r,Percent,100);
 g:=g+muldiv(255-g,Percent,100);
 b:=b+muldiv(255-b,Percent,100);
 result:=RGB(r,g,b);
end;

//******************************************************************************
//    , ""   Angle 
//******************************************************************************
function CreateRotatedFont(F : TFont; Angle : Integer) : hFont;
var LogRec : TLogFont;
begin
  GetObject(F.Handle, SizeOf(LogRec), @LogRec);
  with LogRec do begin
    LogRec.lfEscapement  := Angle * 10;
    LogRec.lfOrientation := 0;
    LogRec.lfOutPrecision := OUT_TT_ONLY_PRECIS;
  end;
  Result := CreateFontIndirect(LogRec);
end;

//******************************************************************************
//   
//    .    ,      
//   .  ,      . 
//         
//  SetGraphicsMode(ACanvas.Handle, GM_ADVANCED);
//******************************************************************************
function DrawZoomStatList(ACanvas : TCanvas; ARect : TRect; 
                          AZoom, ASize : double; AText : string) : boolean;
var lst : TxZoomStatList; //     
    rct : TRect;   
    val : TRect;
    str : string;
    i : Integer;
    p : PxZoomStat;
    wBar : Integer;
//------------------------------------------------------------------------------
    maxTime : Extended;
    maxDiff : Extended;    
    minTime : Extended;
    minDiff : Extended;    
    wTime : Extended;
    wDiff : Extended;    
//--  -------------------------------------------------------------------    
    DC  : HDC;
//    Old : integer;
//    w,h : Integer;
    fnt : hFont;
    tmp : hFont;
//--------------------------------------
    oldM : integer;    
    xFrm : TXForm;
begin
  lst := xGZoomList(false);
  result := CheckParamsValid(ACanvas,ARect,lst,true);
  if not result then exit;

  maxTime :=-1;
  maxDiff :=-1;
  minTime := MaxInt;
  minDiff := MaxInt;

  for i := 0 to lst.Count-1 do begin
    p := lst[i];
    if (p = nil) or (p^.FCount = 0) then continue;
    p^.FTimeC := p^.FTime / p^.FCount;
    p^.FDiffC := p^.FDiff / p^.FCount;  
    if p^.FTimeC > maxTime then maxTime := p^.FTimeC;   
    if p^.FTimeC < minTime then minTime := p^.FTimeC;    
    if p^.FDiffC > maxDiff then maxDiff := p^.FDiffC;   
    if p^.FDiffC < minDiff then minDiff := p^.FDiffC;    
  end;
  
  wTime := (maxTime - minTime) * 0.1;
  minTime := minTime - wTime;
  maxTime := maxTime + wTime;  

  wDiff := (maxDiff - minDiff) * 0.1;
  minDiff := minDiff - wDiff;
  maxDiff := maxDiff + wDiff;  

  with ACanvas do begin
    Font.Height := -trunc(ASize * Font.PixelsPerInch/72);
    wBar := TextWidth('F=0000.00000') div 2; //      
  end;
  //--       -----------------------------
  DC := ACanvas.Handle;
//  w := WidthRect(ARect);
//  h := heightRect(ARect); 
//--  -----------------------------------  
//  Old := SetMapMode(DC, MM_ISOTROPIC);
//  SetWindowExtEx(DC, w, h, nil);
//  SetViewportExtEx(DC, round(AZoom*W), round(AZoom*H), nil);
//---------------------------------------------------

  oldM := SetGraphicsMode(DC, GM_ADVANCED);
  //--   ------------------------------------------------------
  FillChar(xFrm,SizeOf(xFrm),0);  
  //--    -------------------------------------
  xFrm.eM11 := AZoom; //     ,  =1
  xFrm.eM22 := AZoom; //     ,  =1
  xFrm.eDx := lst.FOffX; //   X,     
  xFrm.eDy := lst.FOffY; //   Y,        
  //--    --------------------------------------
  SetWorldTransform(DC, xFrm);

  rct := ARect;
  rct.Top := rct.Top + 10;  
  rct.Bottom := rct.Top + round ( ASize * 190/6.5); //     
  
  if wTime <> 0 then 
    wTime := (rct.Bottom - rct.Top) / (minTime - maxTime);
  if wDiff <> 0 then 
    wDiff := (rct.Bottom - rct.Top) / (minDiff - maxDiff);
  
  try
    with ACanvas do begin
      val := rct;
      val.Left := val.Left + wBar; 
      val.Right := val.Left + wBar;
      Pen.Width := 1;
      
      for i := 0 to lst.Count-1 do begin
        p := lst[i];
        if (p = nil) or (p^.FCount = 0) then continue;
        Pen.Color := Darker(p^.FColor,10);
        //--    -------------------------------
        OffsetRect (val,wBar,0);
        Brush.Color := Lighter(p^.FColor,50);
        val.Top := val.Bottom-round (wTime*(minTime-p^.FTimeC));
        Rectangle(val);
        p^.FTimeR := val;
        //--    --------------------------
        OffsetRect (val,wBar,0);
        Brush.Color := Lighter(p^.FColor,10);
        val.Top := val.Bottom-round (wDiff*(minDiff-p^.FDiffC));        
        Rectangle(val); 
        p^.FDiffR := val;        
        OffsetRect (val,wBar,0);               
      end; 

      for i := 0 to lst.Count-1 do begin
        p := lst[i];
        if (p = nil) or (p^.FCount = 0) then continue;
        Brush.Style := bsClear;
        Font.Color := Darker(p^.FColor,10);
        val := p^.FTimeR;
        str := 't='+FormatFLoat('#0.000#',p^.FTimeC);
        OffsetRect(val,-1,HeightRect(val)+2);
        if lst.GDIPlus then 
          DrawGDIPlusText (ACanvas, val, 0, ASize, str)
        else  
          TextOut (val.Left,val.Top,str);

        Font.Color := Darker(p^.FColor,30); 
        val := p^.FDiffR;
        str := 'f='+FormatFLoat('#0.000#',p^.FDiffC);        
        OffsetRect(val,1,-TextHeight(str)-2);
        if lst.GDIPlus then 
          DrawGDIPlusText (ACanvas, val, 0, ASize, str)
        else  
          TextOut (val.Left, val.Top,str);        

        val := p^.FDiffR;
        str := p^.FName;
        val.Top := val.Bottom+TextHeight(str)+2;
        val.Bottom := ARect.Bottom;

        if lst.GDIPlus then 
          DrawGDIPlusText (ACanvas, val, 30, ASize, str)
        else begin  
          fnt := CreateRotatedFont(Font, -30);
          tmp := SelectObject(DC,fnt);
          try
            //--   -----------------------------------------          
            // SetWindowOrgEx(DC, val.Left,val.Top, nil);
            // SetViewportOrgEx(DC, round(AZoom*val.Left), round(AZoom*val.Top), nil);
            // TextOut (0,0, str);        
            //        ,    
            TextOut (val.Left,val.Top, str);        
          finally
            SelectObject(DC, tmp);
            DeleteObject(fnt);
          end;  
        end;
      end;  
    end;  
  finally
    xFrm.eM11 := 1;
    xFrm.eM22 := 1;
    xFrm.eDx := 0;
    xFrm.eDy := 0;    
    SetWorldTransform(DC, xFrm);
    //--     ---------------------------------------------
    SetGraphicsMode(DC, oldM);
    //--   SetMapMode    -----------------
//    SetMapMode(DC, Old);
  end;
end;

 
end.
