unit utlZoomText;
//******************************************************************************
//       
//******************************************************************************
//        Author : Roman Romanov boss@small-pro.com
//       Project : IP76.RU' 2017
//      Web-site : small-pro.com
//   Description :  ,     
//                 GDrawZoomFuncList -    
//                 GDiffWidth -      
//                               .    
//                                
//******************************************************************************

interface
uses Windows, Classes, SysUtils, Graphics
//-- GDI+ --------------------
   , GDIPAPI
   , GDIPOBJ
   ;

type
//--    ------------------------------------------------------
  TxDrawZoomFunc = function (ACanvas : TCanvas;    //   
                             ARect : TRect;        //  
                             AZoom, ASize : double;// ,  
                             AText : string        //   
                             ) : boolean;          //  
  
var 
//--    ( + TObject(TxDrawZoomFunc)) --------------
  GDrawZoomFuncList : Tstrings = nil;
  GDiffWidth : extended = 0;
  
var 
  GColors : array [0..7] of TColor = (
             clGreen
            ,clRed
            ,clPurple
            ,clTeal
            ,clMaroon
            ,clBlue
            ,clNavy
            ,$00BC6C01
           );
     
//******************************************************************************
//        
//******************************************************************************
function StrToFloatEx (s : string) : double;
//******************************************************************************
//    
//******************************************************************************
function WidthRect  (ARect : TRect) : Integer;
function HeightRect (ARect : TRect) : Integer;
//******************************************************************************
//      
//******************************************************************************
function CheckParamsValid (ACanvas : TCanvas; ARect : TRect; AObject : TObject; AObjChecked : boolean = true) : boolean;
//******************************************************************************
//      ARect    
//******************************************************************************
function CreateBmpRect (ARect : TRect) : TBitmap;
//******************************************************************************
//   -
//******************************************************************************
function DrawZoomGrid (ACanvas : TCanvas; ARect : TRect; AZoom : double) : Integer;
//******************************************************************************
//    GDI+
//******************************************************************************
function DrawGDIPlusText (ACanvas : TCanvas; ARect : TRect; Angle, ASize : double; AText : string; AZoom : double = 1) : boolean;

implementation

//*****************************************************************************
//        
//*****************************************************************************
function StrToFloatEx (s : string) : double;
var r : string;
    i, l : Integer;
    f : boolean;
begin
  s := trim (s);
  l := Length (s);
  r := '';
  f := true;

  for I := 1 to l do begin
    if f and (s[i] in ['.',',']) then begin
      f := false;
      r := r + FormatSettings.DecimalSeparator;
    end;
    if s[i] in ['-','0'..'9','E'] then r := r + s [i];
  end;

  if (r <> '') and (r[1]= FormatSettings.DecimalSeparator) then r := '0' + r;

  result := 0;
  if r = '' then exit;
  if r = '-' then exit;
  try result := StrToFloat (r) except end;
end;

//******************************************************************************
//    
//******************************************************************************
function WidthRect  (ARect : TRect) : Integer;
begin
  result := ARect.Right - ARect.Left;
end;

function HeightRect (ARect : TRect) : Integer;
begin
  result := ARect.Bottom - ARect.Top;
end;

//******************************************************************************
//      
//******************************************************************************
function CheckParamsValid (ACanvas : TCanvas; ARect : TRect; AObject : TObject; AObjChecked : boolean = true) : boolean;
begin
  result := (ACanvas <> nil) and 
            ((not AObjChecked) or (AObject <> nil)) and
            (WidthRect (ARect) > 0) and (HeightRect (ARect)>0);
end;

//******************************************************************************
//      ARect    
//******************************************************************************
function CreateBmpRect (ARect : TRect) : TBitmap;
begin
  result := TBitmap.Create;
  result.Width  := abs (WidthRect (ARect));
  result.Height := abs (HeightRect (ARect));
end;

const
//--       ---------------
  GXOff : double = 5;
  GYOff : double = 5;  
//******************************************************************************
//   -
//******************************************************************************
function DrawZoomGrid (ACanvas : TCanvas; ARect : TRect; AZoom : double) : Integer;
var rct : TRect;
    dvx : double;
    dvy : double;    
    val : Integer; 
    cnt : Integer;
begin
  rct := ARect;
  dvx := GXOff * AZoom;
  dvy := GYOff * AZoom;
  result := 0;
     
  with Acanvas do begin
    val := 0;
    cnt := 0;
    pen.Width := 1;
    Pen.Color := $00E2E2E2;    
    
    while val <= ARect.Right do begin
      cnt := cnt + 1;
      val := round(cnt * dvx);
      MoveTo(ARect.Left, val);
      LineTo(ARect.Right,val);     
    end;

    val := 0;
    cnt := 0;
    while val <= ARect.Right do begin
      cnt := cnt + 1;
      val := round(cnt * dvy);
      if result < 1 then result := val;
      if cnt mod 5 = 0 then begin
        pen.Width := 2;
        Pen.Color := $00DFDFDF;
      end else begin
        pen.Width := 1;
        Pen.Color := $00F2F2F2;        
      end; 
      MoveTo(Arect.Left + val,ARect.Top);
      LineTo(Arect.Left + val,ARect.Bottom);      
    end;
  end;
end;

//******************************************************************************
//        
//******************************************************************************
function DrawZoomCalcRect (ACanvas : TCanvas; ARect : TRect; AZoom, ASize : double; AText : string) : TRect;
var siz : TSize;
begin
  //--    ,   ---------------------------------
  ACanvas.Font.Height := -trunc(ASize * ACanvas.Font.PixelsPerInch/72);
  //--         -----------
  GetTextExtentPoint32(ACanvas.Handle,PWideChar(AText),Length(AText), siz);  
  //----------------------------------------------------------------------------
  //   ,     , 
  //         
  //----------------------------------------------------------------------------  
  result := ARect;
  result.Right := result.Left + round (AZoom * siz.Width);
  result.Bottom := result.Top + round (AZoom * siz.Height);  
end;

//******************************************************************************
//   " "
//  1  = 25.4  = 72  
//******************************************************************************
function DrawZoomTextSimple (ACanvas : TCanvas; ARect : TRect; 
                             AZoom, ASize : double; AText : string) : boolean;
var rct : TRect;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText<>'');
  if not result then exit;
  rct := DrawZoomCalcRect(ACanvas, ARect, AZoom, ASize, AText);
  
  with Acanvas do begin
    Pen.Color := clGreen;
    Pen.Width := 1;
    Rectangle(rct);
    Font.Height := -trunc(AZoom * ASize * Font.PixelsPerInch / 72);
    TextOut (ARect.Left, ARect.Top, AText); 
    GDiffWidth := WidthRect(rct) / TextWidth(AText);
  end;
end;

//******************************************************************************
//   SetGraphicsMode (GM_ADVANCED)
//     
//******************************************************************************
function DrawZoomTextWorldMode(ACanvas : TCanvas; ARect : TRect; 
                               AZoom, ASize : double; AText : string) : boolean;
var rct  : TRect;
    oldM : integer;    
    xFrm : TXForm;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText <> '');
  if not result then exit;
  //--      , =1 -------------
  rct := DrawZoomCalcRect(ACanvas,ARect,1,ASize,AText);
  //--  ""    ----------------------
  oldM := SetGraphicsMode(ACanvas.Handle, GM_ADVANCED);

  try
    //--   ------------------------------------------------------
    FillChar(xFrm,SizeOf(xFrm),0);  
    //--    -------------------------------------
    //   x' = x * eM11 + y * eM21 + eDx 
    //   y' = x * eM12 + y * eM22 + eDy 
    xFrm.eM11 := AZoom;
    xFrm.eM22 := AZoom;
    //--    --------------------------------------
    SetWorldTransform(ACanvas.Handle, xFrm);
    //--  ,        ---------------------
    with Acanvas do begin
      Pen.Color := clRed;
      Pen.Width := 1;
      Rectangle (rct);
      TextOut (rct.Left, rct.Top, AText);
      //--    /   --------
      GDiffWidth := WidthRect(rct)/TextWidth(AText);
    end;
  finally
    //--      --------------------------------
    xFrm.eM11 := 1;
    xFrm.eM22 := 1;
    SetWorldTransform(ACanvas.Handle, xFrm);
    //--     ---------------------------------------------
    SetGraphicsMode(ACanvas.Handle, oldM);
  end;
end;

//******************************************************************************
//  :    SetMapMode/SetWindowExtEx/SetViewportExtEx
//******************************************************************************
function DrawZoomTextMapMode (ACanvas : TCanvas; ARect : TRect; 
                              AZoom, ASize : double; AText : string) : boolean;
var DC  : HDC;
    rct : TRect;   
    Old : integer;
    w,h : Integer;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText <> '');
  if not result then exit;
  //--   ,       ----   
  rct := DrawZoomCalcRect(ACanvas,ARect,1,ASize,AText);
  //--       -----------------------------
  DC := ACanvas.Handle;
  w := WidthRect(ARect);
  h := heightRect(ARect); 
  //--     MM_ISOTROPIC    X  Y 
  //--    (..       )  
  Old := SetMapMode(DC, MM_ISOTROPIC);
  //--     ----------------------
  SetWindowExtEx(DC, w, h, nil);
  //--     ------------------------
  SetViewportExtEx(DC, round(AZoom*W), round(AZoom*H), nil);
  //--  -------------------------------------------------
  try
    with ACanvas do begin
      Pen.Color := clPurple;
      Pen.Width := 1;
      Rectangle(rct);
      TextOut (ARect.Left, ARect.Top, AText); 
      GDiffWidth := WidthRect(rct)/TextWidth(AText);
    end;  
  finally
    SetMapMode(DC, Old);
  end;
end;

//******************************************************************************
//      SetMapMode/SetWindowExtEx/SetViewportExtEx
//       MM_ANISOTROPIC  
//******************************************************************************
function DrawZoomTextMapModeANI(ACanvas : TCanvas; ARect : TRect; 
                                AZoom, ASize : double; AText : string) : boolean;
var DC  : HDC;
    rct : TRect;   
    Old : integer;
    w,h : Integer;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText <> '');
  if not result then exit;

  DC := ACanvas.Handle;
  w := WidthRect(ARect);
  h := heightRect(ARect);  
  rct := DrawZoomCalcRect(ACanvas,ARect,1,ASize,AText);
  //--   MM_ANISOTROPIC    
  //--      
  Old := SetMapMode(DC, MM_ANISOTROPIC);
  //--     ----------------------
  SetWindowExtEx(DC, w, h, nil);
  //--     ------------------------
  SetViewportExtEx(DC, round(AZoom*W), round(AZoom/2*H), nil);
  //--  -------------------------------------------------
  try
    with ACanvas do begin
      Pen.Color := clFuchsia;
      Pen.Width := 1;
      Rectangle(rct);
      TextOut (ARect.Left, ARect.Top, AText); 
      GDiffWidth := WidthRect(rct)/TextWidth(AText);
    end;  
  finally
    SetMapMode(DC, Old);
  end;
end;

//******************************************************************************
//      SetMapMode/SetWindowExtEx/SetViewportExtEx
//   MM_HIENGLISH -      0.001 . 
//******************************************************************************
function DrawZoomTextMapModeHIENGLISH(ACanvas : TCanvas; ARect : TRect; 
                                      AZoom, ASize : double; AText : string) : boolean;
var DC : HDC;
    Old: integer;
    pnt : TPoint;
    rct : TRect;
    siz : TSize;
    tmp : Integer;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText <> '');
  if not result then exit;
  //--    ,     ---------
  ACanvas.Font.Height := -trunc(ASize * ACanvas.Font.PixelsPerInch / 72);
  tmp := ACanvas.Font.Height;
  DC := ACanvas.Handle;
  //--       ------------------------
  pnt.X := GetDeviceCaps(DC,LogPixelsX);
  //--       --------------------------
  pnt.Y := GetDeviceCaps(DC,LogPixelsY);
  //--     (0.001 )----------------------------------- 
  GetTextExtentPoint32(DC,PWideChar(AText),Length(AText), siz);
  rct.Top := -round(1000* AZoom * ARect.Top / pnt.Y);
  rct.Left := round(1000* AZoom * ARect.Left / pnt.X);  
  rct.Right := rct.Left + round(1000* AZoom * siz.Width / pnt.X);
  rct.Bottom := rct.Top - round(1000* AZoom * siz.Height / pnt.Y);  
  
  ACanvas.Font.Height := -round(rct.Bottom-rct.Top) ;
  Old := SetMapMode(DC, MM_HIENGLISH);
  
  try
    with Acanvas do begin
      Pen.Color := clTeal;
      Pen.Width := 1;
      Rectangle (rct);
      TextOut (rct.Left, rct.Top, AText);
      GDiffWidth := WidthRect(rct) / TextWidth(AText);
    end;
  finally
    SetMapMode(DC, Old);
    ACanvas.Font.Height := tmp;
  end;
end;

//******************************************************************************
//    
//******************************************************************************
function DrawZoomTextChar(ACanvas : TCanvas; ARect : TRect; 
                          AZoom, ASize : double; AText : string) : boolean;
var rct : TRect;
    fct : double;
    i : Integer;
    w : Integer;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText <> '');
  if not result then exit;
  //-- ,        ----------
  rct := DrawZoomCalcRect(ACanvas,ARect,AZoom,ASize,AText);

  try
    with ACanvas do begin
      Pen.Color := clMaroon;
      Pen.Width := 1;
      Rectangle(rct);
      GDiffWidth := WidthRect (rct);
      //--   ----------------------------------------------
      Font.Height := -trunc(AZoom * ASize * Font.PixelsPerInch/72);
      //--  ""    ----------------------------
      fct := WidthRect (rct)/TextWidth(AText);
      //--     ,   ,  
      w := 0;      
      for i := 1 to Length(AText) do begin
        TextOut (rct.Left, rct.Top, AText[i]);
        w := w + TextWidth(AText[i]);
        //--        ----- 
        rct.Left := round (ARect.Left + w * fct);
      end; 
      GDiffWidth := GDiffWidth / (rct.Left-ARect.Left);
    end;  
  except
    result := false;
  end;
end;

//******************************************************************************
//     TBitmap  StretchDraw
//******************************************************************************
function DrawZoomTextBitmap(ACanvas : TCanvas; ARect : TRect; 
                            AZoom, ASize : double; AText : string) : boolean;
var rct: TRect; 
    val : TRect; 
    siz: TSize; 
    bmp: TBitmap; // - "" 
    dst: TBitmap; // -stretch 
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText <> '');
  if not result then exit;
  //-- ,        ----------  
  rct := DrawZoomCalcRect(Acanvas,Arect,AZoom,ASize,AText); 
  //--      -----------------------------
  ACanvas.Font.Height := -trunc(AZoom * ASize * ACanvas.Font.PixelsPerInch / 72);
  GetTextExtentPoint32(ACanvas.Handle,PWideChar(AText),Length(AText), siz);
  val := ARect;
  val.Right := val.Left + siz.Width;
  val.Bottom := val.Top + siz.Height;
  //-- -,     -----------------------------------
  bmp := CreateBMPRect (val);//  ,  ""  
  try 
    with bmp.Canvas do begin
      Font.Assign(ACanvas.Font);
      Brush.Color := clWhite;
      TextOut(0,0,AText);
    end;
    //--      ----------------------------------
    dst := CreateBmpRect(rct);
    //-- / ""   ,   
    dst.Canvas.StretchDraw(dst.Canvas.ClipRect,bmp);
    //--      ---------------------------------------
    dst.TransparentColor := clWhite;
    dst.Transparent := true;
    with ACanvas do begin
      Pen.Color := clBlue;
      Pen.Width := 1;
      Rectangle(rct);
      ACanvas.Draw(rct.Left,rct.Top,dst);
    end; 
    GDiffWidth := WidthRect(rct) / dst.Width; 
  finally
    if dst <> nil then dst.Free;
    bmp.Free;
  end;        
end;

//******************************************************************************
//   GDI+    
//******************************************************************************
function DrawZoomTextGDIPlus(ACanvas : TCanvas; ARect : TRect; 
                             AZoom, ASize : double; AText : string) : boolean;
var clr : TColor;
    grp : TGPGraphics;
    brh : TGPSolidBrush;
    nam : TGPFontFamily;
    fsl : FontStyle;
    src : TGPRectF;
    fnt : TGPFont;    
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText<>'');
  if not result then exit;
  ACanvas.Font.Height := -trunc(AZoom * ASize * ACanvas.Font.PixelsPerInch / 72);

  grp := TGPGraphics.Create(ACanvas.Handle);
  try
    with ACanvas do begin
      clr := Font.Color;
      //--    ---------------------------------------------
      nam := TGPFontFamily.Create(Font.Name);
      //--    ---------------------------------------------
      fsl := FontStyleRegular;
      if fsBold in Font.Style then fsl := fsl + FontStyleBold;
      if fsItalic in Font.Style then fsl := fsl + FontStyleItalic;
      if fsUnderline in Font.Style then fsl := fsl + FontStyleUnderline;      
      if fsStrikeOut in Font.Style then fsl := fsl + FontStyleStrikeout;      
      //--    ""    ----
      grp.SetTextRenderingHint(TextRenderingHintAntiAlias);
      //--    ,   -------------------------------
      brh := TGPSolidBrush.Create(MakeColor(GetRValue(clr),GetGValue(clr),GetBValue(clr)));
      //--    ,  ""  ----------------------
      Fnt := TGPFont.Create(nam, ASize * Font.PixelsPerInch / 72, fsl, UnitPixel); 
      //--  ""  -------------------------------
      grp.MeasureString(AText,-1,fnt,MakePoint(ARect.Left*1.0, ARect.Top*1.0),src);
      //--  ""  -------------------------------
      Pen.Color := clNavy;      
      pen.Width := 1;
      Rectangle (round(src.X),round(src.Y),round(src.X + AZoom*src.Width),round(src.Y + AZoom*src.Height));
      //--    ,     -------------------
      GDiffWidth := AZoom*src.Width;
      Fnt.Free;

      //--      -------------------------------------
      Fnt := TGPFont.Create(nam, AZoom * ASize * Font.PixelsPerInch / 72, fsl, UnitPixel); 
      grp.SetTextRenderingHint(TextRenderingHintAntiAlias);
      grp.DrawString(AText, -1, Fnt, MakePoint(ARect.Left*1.0, ARect.Top*1.0), brh);              
      //--        ------------------
      grp.MeasureString(AText,-1,fnt,MakePoint(ARect.Left*1.0, ARect.Top*1.0),src);
      GDiffWidth := GDiffWidth / src.Width;      
    end;  
  except
    result := false;
  end;
  Fnt.free;
  brh.free;
  nam.free;
  grp.free;
end;

//******************************************************************************
//   GDI+    
//******************************************************************************
function DrawZoomTextGDIPlusScale(ACanvas : TCanvas; ARect : TRect; 
                                  AZoom, ASize : double; AText : string) : boolean;
var clr : TColor;
    grp : TGPGraphics;
    brh : TGPSolidBrush;
    nam : TGPFontFamily;
    fsl : FontStyle;
    src : TGPRectF;
    fnt : TGPFont;  
    pnt : TGPPointF;  
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText<>'');
  if not result then exit;

  grp := TGPGraphics.Create(ACanvas.Handle);
  try
    with ACanvas do begin
      clr := Font.Color;
      pnt := MakePoint(ARect.Left*1.0, ARect.Top*1.0); 
      //--    ---------------------------------------------
      nam := TGPFontFamily.Create(Font.Name);
      //--    ---------------------------------------------
      fsl := FontStyleRegular;
      if fsBold in Font.Style then fsl := fsl + FontStyleBold;
      if fsItalic in Font.Style then fsl := fsl + FontStyleItalic;
      if fsUnderline in Font.Style then fsl := fsl + FontStyleUnderline;      
      if fsStrikeOut in Font.Style then fsl := fsl + FontStyleStrikeout;      
      //--    ""    ----
      grp.SetTextRenderingHint(TextRenderingHintAntiAlias);
      //--    ,   -------------------------------
      brh := TGPSolidBrush.Create(MakeColor(GetRValue(clr),GetGValue(clr),GetBValue(clr)));
      //--    ,  ""  ----------------------
      Fnt := TGPFont.Create(nam, ASize * Font.PixelsPerInch / 72, fsl, UnitPixel); 
      //--  ""  -------------------------------
      grp.MeasureString(AText,-1,fnt,pnt,src);
      //--  ""  -------------------------------
      Pen.Color := $00BC6C01;      
      pen.Width := 1;
      Rectangle (round(AZoom*src.X),round(AZoom*src.Y),round(AZoom*(src.X + src.Width)),round(AZoom*(src.Y + src.Height)));
      //--    ----------------------------------
      grp.ScaleTransform(AZoom,AZoom); 
      grp.DrawString(AText, -1, Fnt, pnt, brh);              
      GDiffWidth := 1; 
    end;  
  except
    result := false;
  end;
  Fnt.free;
  brh.free;
  nam.free;
  grp.free;
end;

//******************************************************************************
//    GDI+
//******************************************************************************
function DrawGDIPlusText (ACanvas : TCanvas; ARect : TRect; 
              Angle, ASize : double; AText : string; AZoom : double = 1) : boolean;
var clr : TColor;
    grp : TGPGraphics;
    brh : TGPSolidBrush;
    nam : TGPFontFamily;
    fsl : FontStyle;
    fnt : TGPFont;    
    pnt : TGPPointF;
begin
  result := CheckParamsValid(ACanvas,ARect,nil,false) and (AText<>'');
  if not result then exit;

  grp := TGPGraphics.Create(ACanvas.Handle);
  try
    with ACanvas do begin
      clr := Font.Color;
      //--    ---------------------------------------------
      nam := TGPFontFamily.Create(Font.Name);
      //--    ---------------------------------------------
      fsl := FontStyleRegular;
      if fsBold in Font.Style then fsl := fsl + FontStyleBold;
      if fsItalic in Font.Style then fsl := fsl + FontStyleItalic;
      if fsUnderline in Font.Style then fsl := fsl + FontStyleUnderline;      
      if fsStrikeOut in Font.Style then fsl := fsl + FontStyleStrikeout;      
      //--    ,   -------------------------------
      brh := TGPSolidBrush.Create(MakeColor(GetRValue(clr),GetGValue(clr),GetBValue(clr)));
      //--    ,  ""  ----------------------
      Fnt := TGPFont.Create(nam, ASize * Font.PixelsPerInch / 72, fsl, UnitPixel); 
      //--    ""    ----      
      grp.SetTextRenderingHint(TextRenderingHintAntiAlias);
      //--     --------------------------------------
      pnt := MakePoint(ARect.Left*1.0, ARect.Top*1.0);
      //--    ----------------------------------
      //--  ,  ,      
      grp.TranslateTransform(pnt.X,pnt.y); 
      //--   ,    ------------------      
      if Angle <> 0 then begin        
        //--    ----------------------------------
        grp.RotateTransform(Angle); 
      end;  
      //--      ""  -------------------
      pnt := MakePoint(0.0,0.0);
      //--   ,    ------------------            
      if AZoom <> 1 then begin
        grp.ScaleTransform(AZoom,AZoom); 
      end;
      //--      -------------------------------------
      grp.DrawString(AText, -1, Fnt, pnt, brh);              
    end;  
  except
    result := false;
  end;
  Fnt.free;
  brh.free;
  nam.free;
  grp.free;
end;
  
initialization
  GDrawZoomFuncList := TStringList.Create;
  GDrawZoomFuncList.AddObject(' ', TObject(@DrawZoomTextSimple));
  GDrawZoomFuncList.AddObject('  (SetGraphicsMode)', TObject(@DrawZoomTextWorldMode));        
  GDrawZoomFuncList.AddObject(' (SetMapMode / MM_ISOTROPIC)', TObject(@DrawZoomTextMapMode));  
//  GDrawZoomFuncList.AddObject('  (SetMapMode / MM_ANISOTROPIC)', TObject(@DrawZoomTextMapModeANI));    
  GDrawZoomFuncList.AddObject(' (SetMapMode / MM_HIENGLISH)', TObject(@DrawZoomTextMapModeHIENGLISH));      
  GDrawZoomFuncList.AddObject(' ', TObject(@DrawZoomTextChar));      
  GDrawZoomFuncList.AddObject('Bitmap ', TObject(@DrawZoomTextBitmap));        
  GDrawZoomFuncList.AddObject('GDI+ simple', TObject(@DrawZoomTextGDIPlus));        
  GDrawZoomFuncList.AddObject('GDI+ scale', TObject(@DrawZoomTextGDIPlusScale));          
    
finalization
  FreeAndNil(GDrawZoomFuncList);
  
end.
